package cz.muni.fi.pa165.service;

import cz.muni.fi.pa165.data.entity.TeamMember;
import cz.muni.fi.pa165.event.*;
import cz.muni.fi.pa165.exception.TeamMemberNotFoundException;
import cz.muni.fi.pa165.mapper.TeamMemberMapper;
import cz.muni.fi.pa165.model.TeamMemberDTO;
import cz.muni.fi.pa165.data.respository.TeamMemberRepository;
import jakarta.persistence.EntityNotFoundException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.*;
import org.springframework.kafka.annotation.KafkaListener;
import org.springframework.kafka.core.KafkaTemplate;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.oauth2.server.resource.authentication.BearerTokenAuthentication;
import org.springframework.stereotype.Service;
import org.springframework.web.client.HttpClientErrorException;
import org.springframework.web.client.RestTemplate;

import java.util.List;

@Service
@Slf4j
public class TeamMemberServiceImpl implements TeamMemberService {
    private final TeamMemberRepository teamMemberRepository;

    private final TeamMemberMapper teamMemberMapper;

    private final RestTemplate restTemplate;

    private final KafkaTemplate<String, TeamMemberCreatedEvent> kafkaTemplateCreated;

    private final KafkaTemplate<String, TeamMemberDeletedEvent> kafkaTemplateDeleted;

    @Autowired
    public TeamMemberServiceImpl(TeamMemberRepository teamMemberRepository, TeamMemberMapper teamMemberMapper, RestTemplate restTemplate, KafkaTemplate<String, TeamMemberCreatedEvent> kafkaTemplateCreated, KafkaTemplate<String, TeamMemberDeletedEvent> kafkaTemplateDeleted) {
        this.teamMemberRepository = teamMemberRepository;
        this.teamMemberMapper = teamMemberMapper;
        this.restTemplate = restTemplate;
        this.kafkaTemplateCreated = kafkaTemplateCreated;
        this.kafkaTemplateDeleted = kafkaTemplateDeleted;
    }

    @Override
    public TeamMemberDTO getTeamMemberByID(long id) {
        TeamMember found = teamMemberRepository.findById(id).orElseThrow(
                () -> new TeamMemberNotFoundException(String.format("TeamMember with id %s was not found", id))
        );
        return teamMemberMapper.teamMemberToTeamMemberDTO(found);
    }

    @Override
    public TeamMemberDTO createTeamMember(TeamMemberDTO teamMemberDTO) {
        TeamMember mapped = teamMemberMapper.teamMemberDTOToTeamMember(teamMemberDTO);

        Long bandId = teamMemberDTO.getBandId();

        TeamMember saved = teamMemberRepository.save(mapped);

        var event = new TeamMemberCreatedEvent(saved.getId(), bandId);
        kafkaTemplateCreated.send("teamMemberCreatedTopic", event);

        log.info("Sent team member created event: {}", event);

        return teamMemberMapper.teamMemberToTeamMemberDTO(saved);
    }

    @Override
    public void removeTeamMemberByID(long id) {
        TeamMember found = teamMemberRepository.findById(id).orElseThrow(
                () -> new TeamMemberNotFoundException(String.format("Team Member with id %s was not found", id))
        );
        teamMemberRepository.delete(found);
    }

    @Override
    public TeamMemberDTO updateTeamMemberByID(TeamMemberDTO teamMemberDTO) {
        TeamMember updated = teamMemberRepository.save(teamMemberMapper.teamMemberDTOToTeamMember(teamMemberDTO));
        return teamMemberMapper.teamMemberToTeamMemberDTO(updated);
    }

    @KafkaListener(topics = "proposalAcceptedTopic", autoStartup = "true", groupId = "proposalAcceptedId")
    public void handleProposalAcceptedEvent(ProposalAcceptedEvent proposalAcceptedEvent) {
        log.info("Received proposal accepted event: {}", proposalAcceptedEvent);
        createTeamMember(new TeamMemberDTO(0, "random", proposalAcceptedEvent.getBandId(), proposalAcceptedEvent.getUserId()));
    }

    @KafkaListener(topics = "bandDeletedTopic", autoStartup = "true", groupId = "bandDeletedId")
    public void handleBandDeletedEvent(BandDeletedEvent bandDeletedEvent) {
        log.info("Received band deleted event: {}", bandDeletedEvent);
        List<TeamMember> affectedMembers = teamMemberRepository.findAllByBandId(bandDeletedEvent.getBandId());
        affectedMembers.forEach(
                teamMember -> removeTeamMemberByID(teamMember.getId())
        );
    }

    @KafkaListener(topics = "userDeletedTopic", autoStartup = "true", groupId = "userDeletedId")
    public void handleUserDeletedEvent(UserDeletedEvent userDeletedEvent) {
        log.info("Received user deleted event: {}", userDeletedEvent);
        List<TeamMember> affectedMembers = teamMemberRepository.findAllByUserId(userDeletedEvent.getUserId());
        affectedMembers.forEach(
                teamMember -> {
                    removeTeamMemberByID(teamMember.getId());
                    var event = new TeamMemberDeletedEvent(teamMember.getId(), teamMember.getBandId());
                    kafkaTemplateDeleted.send("teamMemberDeletedTopic", event);
                    log.info("Sent team member deleted event: {}", event);
                }
        );
    }
}
